#include <stdio.h>
#include <string.h>

#include "Bitmap.h"

void Bitmap::loadPPM (char *filename)
{
	FILE *file = fopen (filename, "r");
	if (!file)
		return;

	char line[128];
	fgets (line, 127, file);
	if (strcmp (line, "P6") != 0) {
		fprintf (stderr, "unsupported PPM format\n");
		return;
	}

	// skip comments
	fgets (line, 127, file);
	while (line[0] == '#')
		fgets (line, 127, file);

	unsigned int i, size;
	sscanf (line, "%d %d", &width, &height);
	fgets (line, 127, file);
	sscanf (line, "%d", &depth);
	if (depth == 255)
		depth = 1;
	else if (depth == 65355)
		depth = 2;
	else {
		fprintf (stderr, "unknown color depth");
		return;
	}

	size = width*height*depth;
	data = new char[size];
	for (i=0; i<size; i++) {
		data[i] = fgetc (file);
	}

	fclose (file);
}

void Bitmap::loadBMP (char *filename)
{
	FILE *file = fopen (filename, "r");
	if (!file)
		return;

	header = new BITMAPINFOHEADER;
	fread (header, sizeof (BITMAPINFOHEADER), 1, file);

	data = new char[header->biSizeImage];
	fread (data, sizeof (char), header->biSizeImage, file);

	fclose (file);
}

void* Bitmap::getData ()
{
	return (void *)data;
}

unsigned int Bitmap::getWidth ()
{
	return width;
}

unsigned int Bitmap::getHeight ()
{
	return height;
}

unsigned int Bitmap::getDepth ()
{
	return depth;
}

void Bitmap::release ()
{
	if (data)
		delete [] data;
	if (header)
		delete header;
	width = height = depth = 0;
}

void Bitmap::getBitmapInfo (BITMAPINFO **ppBitmapInfo, unsigned long *pBitmapInfoBytes)
{
}
