/*
+-----------------------------------------------------------------------------+
| @BEGIN_COPYRIGHT                                                            |
|                                                                             |
| Copyright (C) 2001-2003, The Regents of the University of California        |
| All Rights Reserved.                                                        |
|                                                                             |
| Visualization and Interactive Systems Group                                 |
| University of California, Irvine                                            |
| Irvine, CA 92697                                                            |
|                                                                             |
| Commercial use of this program without express permission of the            |
| University of California, Irvine, is strictly prohibited. Information about |
| usage and redistribution, and a disclaimer of all warrenties are            |
| available in the Copyright file provided with this code.                    |
|                                                                             |
| Developed By:                                                               |
|    Gautam Chaudhary    {gchaudha@uci.edu}                                   |
|    Tara C. Hutchinson  {thutchin@uci.edu}                                   |
|    Falko Kuester       {fkuester@uci.edu}                                   |
|    Thomas Wishgoll     {twischgo@uci.edu}                                   |
|                                                                             |
| @END_COPYRIGHT                                                              |
+-----------------------------------------------------------------------------+
|                                               
| Component  : DibSectn
| Filename   : DibSectn.h
| Sourcefile : DibSectn.cpp
| Language   : C++
|                                               
+----------------------------- Revision Information---------------------------+
| $Author: wischgol $
| $Revision: 1.3 $
| $Date: 2003/09/25 15:29:40 $
| $RCSfile: DIBSECTN.H,v $
+-----------------------------------------------------------------------------+
*/

/*! \file	DIBSECTN.H  
	\brief	Interface for the DIBSections functions

	Company:  Basler Vision Technologies
	Section:  Vision Components
	Project:  1394 Driver
	$Author: wischgol $
	$Revision: 1.3 $
	$Date: 2003/09/25 15:29:40 $

*/

#pragma once


//! Creates a DIBSection based on the criteria passed in as parameters.
HBITMAP DSCreateDIBSection( LONG dwX, LONG dwY, WORD wBits, BOOL Monochrome );

//! Creates a DIBSection based on the contents of a BMP file.
BOOL DSLoadDIBSectionFromBMPFile( LPCTSTR szFileName, HBITMAP *phBitmap, HPALETTE *phPalette );

//! Stores the DIBSection in a BMP file.
BOOL DSStoreDIBSectionInBMPFile( LPCTSTR szFileName, HBITMAP hBitmap );

//! Draws a DIBSection on a target DC.
BOOL DSDrawDIBSectionOnDC( HDC hDC, HBITMAP hBitmap, LPRECT pRect );

//! Creates a palette to be used to display the DIBSection on a palettized display device.
HPALETTE DSCreatePaletteForDIBSection( HBITMAP hBitmap );

//! Allocates and returns a pointer to a BITMAPINFO for the DIBSection.
LPBITMAPINFO DSGetBITMAPINFOForDIBSection( HBITMAP hBitmap );

//! Calculates and returns the size, in bytes, of the color table for the DIB described by the BITMAPINFO.
DWORD DSColorTableSize( LPBITMAPINFO pbmi );

//! Calculates and returns the size, in bytes, of the entire CF_DIB bitmap described by the BITMAPINFO 
//! - including header, color table, and image bits.
DWORD DSTotalBytesSize( LPBITMAPINFO pbmi );

//! Calculates and returns the size, in bytes, of the image bits for the DIB described by the BITMAPINFO.
DWORD DSImageBitsSize( LPBITMAPINFO pbmi );

//! Returns a pointer to the DIBSection's image bits.
LPBYTE DSGetPointerToDIBSectionImageBits( HBITMAP hBitmap );

//! This function will build a palette with a spectrum of colors.  
HPALETTE DSCreateSpectrumPalette( void );
