
#include "stdafx.h"

#include <vfw.h>
#include "DIBSectionLite.h"

#include <iostream>
#include "createvideo.h"

using namespace std;

void createvideo (char *base,
		  char *suffix,
		  int digits,
		  int start,
		  int number,
#ifdef COMPRESSION
		  int keyframe,
#endif
		  int frameRate,
		  char *outFileName)
{
  // only monochrome, colored images, and edge detection supported so far

  char filename[256];
  char strpattern[16];
  PAVIFILE        outfile; // AVI output file
  PAVISTREAM      outstream; // output AVI stream
  HRESULT         hr; // error code
  AVISTREAMINFO   strhdr; // stream header of input AVI file
  LONG            lStreamSize; // frame number
  CDIBSectionLite lpOld; // original image of AVI frame

  // compress options for video stream (if selected)
  AVICOMPRESSOPTIONS CompressionOptions;

  cout << "Creating video " << outFileName << "..." << endl;

  // Allocate memory for the bitmaps.
  sprintf (strpattern, "%%s%%0%dd%%s", digits);
  sprintf (filename, strpattern, base, start, suffix);
  lpOld.Load (_T(filename));

  CoInitialize (NULL);	
  
  // Create new AVI file using AVIFileOpen. 
  hr = AVIFileOpen(&outfile, outFileName, OF_WRITE | OF_CREATE, NULL);
  if (hr != 0) {
    char sz[1024];
    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, hr, 0,sz, 1024, 0))
      cerr << "Cannot open output file: " << sz << endl;
    else
      cerr << "Cannot open output file" << endl;
    lpOld.DeleteObject ();
    return;
  }

  ::ZeroMemory (&strhdr, sizeof (strhdr));
  strhdr.fccType = streamtypeVIDEO;
  strhdr.fccHandler = 0;
  strhdr.dwScale = 1;
  strhdr.dwRate = frameRate;

  // Create a stream using AVIFileCreateStream. 
  hr = AVIFileCreateStream(outfile, &outstream, &strhdr);
  if (hr != 0) {            //Stream created OK? If not, close file. 
    char sz[1024];
    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, hr, 0,sz, 1024, 0))
      cerr << "Could not create output stream: " << sz << endl;
    else
      cerr << "Could not create output stream" << endl;
    lpOld.DeleteObject ();
    return;
  } 

#ifdef COMPRESSION
  AVICOMPRESSOPTIONS FAR * pCompressionOptions[1] = {&CompressionOptions};
  ::ZeroMemory(&CompressionOptions, sizeof(CompressionOptions));

  // Let the user choose the compression format (this will show a message box!)
  if (!AVISaveOptions(NULL, 0, 1, &outstream, (LPAVICOMPRESSOPTIONS FAR *) pCompressionOptions)) {
    cerr << "No compression selected" << endl;
    AVIStreamRelease(outstream);
    AVIFileRelease (outfile);
    lpOld.DeleteObject ();
    return;
  }

  // Handle for the raw video stream
  PAVISTREAM hCompressedVideoStream;

  hr = AVIMakeCompressedStream(
			       &hCompressedVideoStream,
			       outstream,
			       &CompressionOptions, 
			       NULL);
  if (hr != AVIERR_OK) {
    cerr << "ERROR: cannot create compressed stream" << endl;
    AVIStreamRelease(outstream);
    AVIFileRelease (outfile);
    lpOld.DeleteObject ();
    return;
  }

  // We need only the handle of the compressed stream so close the handle 
  // of the raw stream
  AVIStreamClose(outstream);
  outstream = hCompressedVideoStream;
#endif // COMPRESSION

  // Set the stream format (= the format of the DIBs)
  unsigned long BitmapInfoBytes;
  BITMAPINFO *pBitmapInfo = lpOld.GetBitmapInfo();
  BitmapInfoBytes = sizeof(BITMAPINFO)
	  + lpOld.GetColorTableSize ();

  hr = AVIStreamSetFormat(outstream, 0, pBitmapInfo, BitmapInfoBytes);
  if (hr != 0) { 
    char sz[1024];
    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, hr, 0,sz, 1024, 0))
      cerr << "Cannot set format of new stream: " << sz << endl;
    else
      cerr << "Cannot set format of new stream" << endl;
    AVIStreamRelease(outstream);
    AVIFileRelease (outfile);
    lpOld.DeleteObject ();
    return;
  }

  // Read the stream data using AVIStreamRead. 
  for (lStreamSize = 0; lStreamSize < number; lStreamSize++) {
    sprintf (filename, strpattern, base, lStreamSize + start, suffix);
    lpOld.Load (_T(filename));

    // Save the compressed data using AVIStreamWrite. 
    hr = AVIStreamWrite(outstream,
			lStreamSize,
			1,
			lpOld.GetDIBits (),
    			pBitmapInfo->bmiHeader.biSizeImage,
#ifdef COMPRESSION
			NULL,
//			(lStreamSize % keyframe == 0) ? AVIIF_KEYFRAME : NULL,
#else
			AVIIF_KEYFRAME,
#endif
			NULL,
			NULL);
    if (hr != 0) {
      char sz[1024];
      if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, hr, 0,sz, 1024, 0))
	cerr << "Cannot write to output stream: " << sz << endl;
      else
	cerr << "Cannot write to output stream" << endl;
    }

    cout << "Storing frame " << lStreamSize << "..." << endl;
  } 

  AVIStreamRelease(outstream);
  AVIFileRelease (outfile);

  lpOld.DeleteObject ();

  cout << "Video finished." << endl;

  CoUninitialize ();
}
